# sync to the awesome tmb kernel builds. He is the best of them all

###
# Definitions
###
%define major	5
%define sublevel 10
%define stabrel	216
%define buildrel 1
%define codename LTS

# Don't automatically clean up build dirs post building (sometimes useful for debugging)
%define dontclean	0

###
# Lets deal with our options and the resulting kernel 'appends' and names
###
%define bname		kernel
%define kname		%{bname}
%define kappend		pclos

%define tar_vers 	%{major}.%{sublevel}.%{stabrel}
%define realrelease	%{kappend}%{buildrel}
%define rpmversionn 	1
%define rpmrelease 	%mkrel 1

%if %stabrel
%define realversion	%{major}.%{sublevel}.%{stabrel}
%define patches_ver	%{major}.%{sublevel}.%{stabrel}-%{kappend}%{buildrel}
%else
%define realversion	%{major}.%{sublevel}
%define patches_ver	%{major}.%{sublevel}-%{kappend}%{buildrel}
%endif

%define kverrel 	%{realversion}-%{realrelease}

%define top_dir_name 	%{kname}-build
%define build_dir 	${RPM_BUILD_DIR}/%{top_dir_name}
%define src_dir 	%{build_dir}/linux-%{tar_vers}

###
# Our various kernel build definitions
###
%define build_source 0
%define build_doc	0
%define build_k8	0
%define build_server 0
%define build_linus	0
%define build_bfs	0
%define build_bfs_pae 0


%{?_without_source: %global build_source 0}
%{?_without_doc: %global build_doc 0}
%{?_without_server: %global build_server 0}
%{?_without_k8: %global build_k8 0}
%{?_without_linus: %global build_linus 0}
%{?_without_bfs: %global build_bfs 0}
%{?_without_bfs_pae: %global build_bfs_pae 0}

%{?_with_source: %global build_source 1}
%{?_with_doc: %global build_doc 1}
%{?_with_server: %global build_server 1}
%{?_with_k8: %global build_k8 1}
%{?_with_linus: %global build_linus 1}
%{?_with_bfs: %global build_bfs 1}
%{?_with_bfs_pae: %global build_bfs_pae 1}

# various defines for build options
%define preempt_full 1
%define preempt_vol	0
%define k8ops		0
%define pae_on		0
%define hz_100		0
%define hz_250		0
%define xen_guest	0

%if %(if [ -z "$CC" ] ; then echo 0; else echo 1; fi)
%define kmake %make_build CC="$CC"
%else
%define kmake %make_build
%endif
# there are places where parallel make don't work
%define smake make

# Parallelize xargs invocations on smp machines
%define kxargs xargs %([ -z "$RPM_BUILD_NCPUS" ] \\\
	&& RPM_BUILD_NCPUS="`/usr/bin/getconf _NPROCESSORS_ONLN`"; \\\
	[ "$RPM_BUILD_NCPUS" -gt 1 ] && echo "-P $RPM_BUILD_NCPUS")


# Alias for x86_64 builds
%define target_arch	%(echo %{_arch} | sed -e "s/amd64/x86_64/")

###
# Our various kernel builds
###
#default kernel summary
%define summary The Linux kernel

# i686 server kernel
%if %build_server
%define build_source	0
%define build_doc	0
%define summary		Linux kernel (pae enabled)
%define kname 		%{bname}
%define realrelease 	%{kappend}%{buildrel}.pae
%define pae_on		1
%endif

# linus kernel
%if %build_linus
#i686 build
%define summary	Linux kernel (no extra patches)
%define summary	'Linus' Linux kernel (no extra patches) for %{_arch} arch
%define kname 		kernel-linus
%define kverrel 	%{realversion}
%define hz_250		1
%endif

# AMD64 / Opteron x86 build K8 optimised kernel
%if %build_k8
%define build_source	0
%define build_doc	0
%define summary		AMD64/Opteron (K8) optimised linux kernel for x86 arch
%define kname 		%{bname}
%define optflags 	%__common_cflags_with_ssp -fomit-frame-pointer -march=athlon64 -fasynchronous-unwind-tables
%define realrelease 	%{kappend}%{buildrel}.a64
%define k8ops		1
%endif

# bfs kernel
%if %build_bfs
%define build_source	0
%define build_doc	0
%define summary		Linux kernel with the BFS scheduler
%define kname 		%{bname}
%define realrelease 	%{kappend}%{buildrel}.bfs
%define preempt_vol 0
%define preempt_full 1
%endif

#arrrgh my brain!
%if %build_bfs_pae
%define build_source	0
%define build_doc	0
%define summary		Linux kernel pae bfs scheduler
%define kname 		%{bname}
%define realrelease 	%{kappend}%{buildrel}.pae.bfs
%define preempt_vol	0
%define preempt_full 1
%define pae_on		1
%endif


###
# Generic rpm defines
###
Summary: 		%{summary}
Name: 			%{kname}-%{kverrel}
Version:		%{rpmversionn}
Release:		%{rpmrelease}
License: 		GPL
Group:			System/Kernel and hardware
ExclusiveArch: 	%{ix86} x86_64
URL: 			http://www.kernel.org/

###
# Deal with our sources
###
# Do you want the src.rpm to build with no kernel source?
%define build_nosrc 1
%{?_with_nosrc: %global build_nosrc 1}
# Due to nosrc.rpm we should check for and if required download the appropriate kernel sources
%(if [ ! -f %_sourcedir/linux-%{tar_vers}.tar.xz ]; then cd %_sourcedir/ && wget http://www.kernel.org/pub/linux/kernel/v5.x/linux-%{tar_vers}.tar.xz; fi)
# This is for full SRC RPM
Source0: linux-%{tar_vers}.tar.xz
Source1: config-x86_64
# This is for disabling *config, mrproper, prepare, scripts on -devel rpms
Source2: 	disable-mrproper-in-devel-rpms.patch

# This is for stripped SRC RPM
%if %build_nosrc
Nosource: 0
%endif
Source99: build-kernels.sh

###
# Generic rpm Requires and BuildRequires
###
BuildRoot:	%{_tmppath}/%{name}-build
Provides:	kernel = %{kverrel}, %{kname} = %{kverrel}
Provides:	alsa = 1.0.24
Provides:	should-restart = system
Requires:   mkinitrd
Requires:   bootloader-utils
Requires:   sysfsutils
Requires:   kernel-firmware
Requires:   lz4
Requires:   zstd
Requires:   xz
Requires:	glibc
Requires:   flex
Requires:   bison
Requires:   glibc
Requires:  kmod
Requires:  libnotify

BuildRequires: build-essentials
BuildRequires: gcc
BuildRequires: module-init-tools
BuildRequires: wget
BuildRequires: flex
BuildRequires: bison
BuildRequires: lib64zstd-devel
BuildRequires: lib64lzma-devel
BuildRequires: lz4-devel
BuildRequires: lib64bzip2-devel
BuildRequires: lib64elfutils-devel
BuildRequires: lib64openssl-devel
BuildRequires: pciutils-devel
BuildRequires: build-essentials
BuildRequires: kmod
BuildRequires: lib64kmod-devel

Requires:	%{kname}-devel = %{kverrel}
Provides:	dkms-lzma
Provides:	dkms-squashfs-lzma

#-------------------------------------
Conflicts:  dkms-nvidia304
Conflicts:  nvidia304-cuda-opencl
Conflicts:  nvidia304-devel
Conflicts:  nvidia304-doc-html
Conflicts:  x11-driver-video-nvidia304
#-------------------------------------
Patch1000:	x86-pci-toshiba-equium-a60-assign-busses.patch
Patch1001:	x86-boot-video-80x25-if-break.patch
Patch1005:	x86-default_poweroff_up_machines.patch
Patch1010:	x86-increase-default-minimum-vmalloc-area-by-64MB-to-192MB.patch
Patch1015:	Revert-cpufreq-pcc-Enable-autoload-of-pcc-cpufreq-fo.patch
Patch1032:	x86-disable-HPET-on-Intel-Coffee-Lake-Refresh-platforms.patch
Patch1035:	revert-module-delay-kobject-uevent-until-after-module-init-.patch
Patch1100:	acpi-CLEVO-M360S-disable_acpi_irq.patch
Patch1105:	acpi-processor-M720SR-limit-to-C2.patch
Patch1110:	ACPI-video-Add-a-quirk-to-force-acpi-video-backlight.patch
Patch1200:	scsi-megaraid-new-sysfs-name.patch
Patch1205:	ide-pci-sis5513-965.patch
Patch1215:	dm-raid-aliases.patch
Patch1220:	block-floppy-disable-pnp-modalias.patch
Patch1225:	ata-prefer-ata-drivers-over-ide-drivers-when-both-are-built.patch
Patch1231:	revert-block-remove-check_disk_change.patch
Patch1251:	crypto-aesni-implement-support-for-cts-cbc-aes.patch
Patch1253:	crypto-x86-aes-ni-xts-rewrite-and-drop-indirections-.patch
Patch1310:	fs-aufs5.10.patch
Patch1311:	fs-aufs5.10-symbol-exports.patch
Patch1312:	fs-aufs5.10-tristate.patch
Patch1400:	firewire-ieee1394-module-aliases.patch
Patch1500:	drm-amdgpu-fix-the-issue-that-apu-has-no-smu-firmwar.patch
Patch1501:	drm-amd-amdgpu-set-the-default-value-of-noretry-to-1.patch
Patch1502:	drm-amdgpu-protect-eeprom-update-from-GPU-reset.patch
Patch1503:	drm-amd-amdgpu-enable-noretry-for-Sienna_Cichlid-Nav.patch
Patch1504:	drm-amdgpu-disable-rom-clock-gating-support-for-APUs.patch
Patch1505:	drm-amdgpu-increase-reserved-VRAM-size-to-8MB.patch
Patch1506:	drm-amdgpu-add-missing-sienna-cichlid-did.patch
Patch1508:	drm-amdgpu-fix-the-issue-that-retry-constantly-once-.patch
Patch1509:	drm-amdkfd-fix-null-pointer-panic-while-free-buffer-.patch
Patch1510:	drm-amdgpu-set-default-value-of-noretry-to-1-for-spe.patch
Patch1511:	drm-amdgpu-default-noretry-0-for-navi1x-and-newer-v2.patch
Patch1520:	drm-amdgpu-SI-and-CIK-enabled-by-default.patch
Patch1530:	char-agp-intel-new-Q57-id.patch
Patch1550:	Revert-drm-radeon-simplify-and-cleanup-setting-the-d.patch
Patch1551:	Revert-drm-radeon-handle-PCIe-root-ports-with-addres.patch
Patch1552:	i2c_nvidia_gpu-change-err-into-info.patch
Patch1560:	drm-nouveau-core-recognise-GA10-024.patch
Patch1561:	drm-nouveau-pci-ga10-024-initial-support.patch
Patch1562:	drm-nouveau-bios-ga10-024-initial-support.patch
Patch1563:	drm-nouveau-devinit-ga10-024-initial-support.patch
Patch1564:	drm-nouveau-mc-ga10-024-initial-support.patch
Patch1565:	drm-nouveau-privring-ga10-024-initial-support.patch
Patch1566:	drm-nouveau-imem-ga10-024-initial-support.patch
Patch1567:	drm-nouveau-fb-ga10-024-initial-support.patch
Patch1568:	drm-nouveau-timer-ga10-024-initial-support.patch
Patch1569:	drm-nouveau-mmu-ga10-024-initial-support.patch
Patch1570:	drm-nouveau-bar-ga10-024-initial-support.patch
Patch1571:	drm-nouveau-gpio-ga10-024-initial-support.patch
Patch1572:	drm-nouveau-i2c-ga10-024-initial-support.patch
Patch1573:	drm-nouveau-dmaobj-ga10-24-initial-support.patch
Patch1574:	drm-nouveau-disp-ga10-24-initial-support.patch
Patch1600:	hwmon-add-Corsair-PSU-HID-controller-driver.patch
Patch1601:	hwmon-corsair-psu-fix-unintentional-sign-extension-issue.patch
Patch1602:	hwmon-corsair-psu-Fix-fan-rpm-calculation.patch
Patch1610:	hwmon-amd_energy-Add-AMD-family-19h-model-01h-x86-match.patch
Patch1621:	hwmon-k10temp-add-Ryzen-5xxx-support.patch
Patch1720:	hid-0001-SFH-Add-maintainers-and-documentation-for-AMD-SFH-ba.patch
Patch1721:	hid-0002-SFH-PCIe-driver-to-add-support-of-AMD-sensor-fusion-.patch
Patch1722:	hid-0003-SFH-Transport-Driver-to-add-support-of-AMD-Sensor-Fu.patch
Patch1723:	hid-0004-SFH-Create-HID-report-to-Enable-support-of-AMD-senso.patch
Patch1724:	hid-0005-AMD_SFH-Fix-for-incorrect-Sensor-index.patch
Patch1725:	hid-0006-HID-SFH-Add-documentation.patch
Patch1726:	hid-0007-SFH-fix-error-return-check-for-ERESTARTSYS.patch
Patch1727:	hid-sfh-fix-address-space-confusion.patch
Patch1900:	net-sis190-fix-list-usage.patch
Patch1910:	net-netfilter-IFWLOG.patch
Patch1911:	net-netfilter-IFWLOG-mdv.patch
Patch1912:	net-netfilter-IFWLOG-2.6.35-buildfix.patch
Patch1913:	net-netfilter-IFWLOG-2.6.37-buildfix.patch
Patch1914:	net-ipv4-netfilter-ipt_IFWLOG-3.6-buildfix.patch
Patch1915:	net-netfilter-IFWLOG-3.7-buildfix.patch
Patch1916:	net-netfilter-IFWLOG-remove-unused-label.patch
Patch1917:	net-netfilter-ipt_IFWLOG-4.12-buildfix.patch
Patch1918:	net-netfilter-IFWLOG-5.0-buildfix.patch
Patch1920:	net-netfilter-psd.patch
Patch1921:	net-netfilter-psd-mdv.patch
Patch1922:	net-netfilter-psd-2.6.35-buildfix.patch
Patch1923:	net-netfilter-psd-fix-redefines.patch
Patch1930:	net-wireless-rtw88-add-r8822be-alias.patch
Patch1931:	net-wireless-rtw88-add-8723de-alias.patch
Patch1933:	net-wireless-rtw88-8821c-Correct-CCK-RSSI.patch
Patch1934:	net-wireless-rtw88-8821c-support-RFE-type2-wifi-NIC.patch
Patch1940:	net-wireless-rtw89-add-CAM-files.patch
Patch1941:	net-wireless-rtw89-add-BT-coexistence-files.patch
Patch1942:	net-wireless-rtw89-add-core-and-trx-files.patch
Patch1943:	net-wireless-rtw89-add-debug-files.patch
Patch1944:	net-wireless-rtw89-add-efuse-files.patch
Patch1945:	net-wireless-rtw89-add-files-to-download-and-communicate-with-fir.patch
Patch1946:	net-wireless-rtw89-add-MAC-files.patch
Patch1947:	net-wireless-rtw89-implement-mac80211-ops.patch
Patch1948:	net-wireless-rtw89-add-pci-files.patch
Patch1949:	net-wireless-rtw89-add-phy-files.patch
Patch1950:	net-wireless-rtw89-define-register-names.patch
Patch1951:	net-wireless-rtw89-add-regulatory-support.patch
Patch1952:	net-wireless-rtw89-8852a-add-8852a-specific-files.patch
Patch1953:	net-wireless-rtw89-8852a-add-8852a-RFK-files.patch
Patch1954:	net-wireless-rtw89-8852a-add-8852a-RFK-tables.patch
Patch1955:	net-wireless-rtw89-8852a-add-8852a-tables.patch
Patch1956:	net-wireless-rtw89-add-ser-to-recover-error-reported-by-firmware.patch
Patch1957:	net-wireless-rtw89-add-Kconfig-and-Makefile.patch
Patch1960:	net-wireless-iwlwifi-avoid-endless-HW-errors-at-assert-time.patch
Patch1962:	net-wireless-iwlwifi-iwlwifi-dbg-Mark-ucode-tlv-data-as-const.patch
Patch1990:	pci-decode-pcie-64-gt-link-speed.patch
Patch2000:	platform-x86-add-shuttle-wmi-driver.patch
Patch2001:	platform-x86-shuttle-wmi-drop-devinit-exit.patch
Patch2002:	platform-x86-shuttle-wmi-4.2-buildfix.patch
Patch2003:	platform-x86-shuttle-wmi-4.13-buildfix.patch
Patch2004:	platform-x86-shuttle-wmi-kernel-5.5.patch
Patch2280:	thermal-intel_pch_thermal-Add-PCI-ids-for-Lewisburg-PCH.patch
Patch2300:	hid-usbhid-IBM-BladeCenterHS20-quirk.patch
Patch2310:	usb-storage-unusual_devs-add-id.patch
Patch2311:	usb-storage-unusual_devs-add-id-2.6.37-buildfix.patch
Patch2400:	media-usb-pwc-lie-in-proc-usb-devices.patch
Patch2501:	video-console-vgacon-treat-nokmsboot-as-nomodeset.patch
Patch2550:	KVM-squelch-uninitialized-variable-warning.patch
Patch2600:	tools-testing-selftest-Makefile-remove-powerpc-reference.patch
Patch2605:	tools-perf-ui-include-slang.patch
Patch2650:	2650-kconfig-qconf-drop-Qt4-support.patch
Patch2651:	2651-kconfig-qconf-use-a-variable-to-pass-packages-to-pkg.patch
Patch2652:	2652-kconfig-qconf-convert-to-Qt5-new-signal-slot-connect.patch
Patch2653:	2653-kconfig-make-lkc.h-self-sufficient-include-wise.patch
Patch2654:	2654-kconfig-qconf-show-Qt-version-in-the-About-dialog.patch
Patch2655:	2655-kconfig-clean-up-header-inclusion.patch
Patch2657:	2657-kconfig-doc-fix-fileno-to-filename.patch
Patch2700:	uapi-avoid-namespace-conflict-in-linux-posix_types.h.patch
Patch3000:	3rd-3rdparty-tree.patch
Patch3001:	3rd-3rdparty-merge.patch
Patch3100:	3rd-ndiswrapper-1.63.patch
Patch3101:	3rd-ndiswrapper-Kconfig.patch
Patch3102:	3rd-ndiswrapper-Makefile-build-fix.patch
Patch3103:	3rd-niswrapper-Kconfig-Makefile.patch

# rtl8812au https://github.com/aircrack-ng/rtl8812au
Patch3200:	3rd-rtl8812au_afd38cc.patch
Patch3201:	3rd-rtl8812au-Kconfig-Makefile.patch
Patch3202:	3rd-rtl8812au-rename.patch

Patch3300:	3rd-rtl8723de.patch
Patch3301:	3rd-rtl8723de-Kconfig-Makefile.patch
Patch3302:	3rd-rtl8723de-fix-redefine.patch
Patch3303:	3rd-rtl8723de-nodebug.patch
Patch3304:	3rd-rtl8223de-kernel-5.6.patch
Patch3305:	3rd-rtl8723de-add-kernel-5.8-support.patch
Patch3400:	3rd-viahss-0.92.patch
Patch3401:	3rd-viahss-config.patch
Patch3402:	3rd-viahss-module-license.patch
Patch3403:	3rd-viahss-2.6.35-buildfix.patch
Patch3404:	3rd-viahss-3.0-buildfix.patch
Patch3405:	3rd-viahss-Kconfig-Makefile.patch

# rtl8821 https://github.com/tomaspinho/rtl8821ce
Patch3500:	3rd-rtl8821ce_50c1b12.patch
Patch3501:	3rd-rtl8821ce-Kconfig-Makefile.patch

Patch3504:  3rd-fix-kconfig.patch
Patch5002:	CVE-2019-12379.patch
Patch5003:	CVE-2020-16119-DCCP-CCID-structure-use-after-free.patch
Patch7000:  0001-scsi-smartpqi-add-inspur-advantech-ids.patch
Patch7001:  0001-Vulcan-AHCI-PCI-bar-fix-for-Broadcom-Vulcan-early-si.patch
Patch7002:  mpt-vmware-fix.patch
Patch7003:  tex-uterror.c.patch
Patch7004:  tex-utxferror.c.patch
Patch7006:  tex-uksm-5.10.patch
Patch7010:  tex-no-irq-handler-for-vector.patch

###
# Kernel rpm descriptions
###
%description
This is a LTS kernel for PCLinuxOS.

%if %{build_server}
%description
This is a LTS kernel for PCLinuxOS.
%endif

%if %{build_linus}
%description
This is a LTS kernel for PCLinuxOS.
%endif

%if %{build_k8}
%description
This is a LTS kernel for PCLinuxOS.
%endif

%if %{build_bfs}
%description
This is a LTS kernel for PCLinuxOS.
%endif

%if %{build_bfs_pae}
%description
This is a LTS kernel for PCLinuxOS.
%endif


###
# kernel source packages:
###
%if %build_source
%package -n	%{kname}-source-%{kverrel}
Version:	%{rpmversionn}
Release:	%{rpmrelease}
Provides:	kernel-source = %{kverrel}
Requires:	glibc-devel, ncurses-devel, make, gcc, perl
Summary:	The %{kname}-%{kverrel} source code.
Group:		System/Kernel and hardware
AutoReqProv:	no
%ifarch %{ix86}	
Conflicts:	arch(x86_64)
%endif

%description -n %{kname}-source-%{kverrel}
The kernel-source package contains the source code files for the Linux kernel.
These source files are needed to build most C programs, since
they depend on the constants defined in the source code. The source
files can also be used to build a custom kernel that is better tuned to
your particular hardware, if you are so inclined (and you know what you're
doing).
%endif

%package -n	%{kname}-devel-%{kverrel}
Version:	%{rpmversionn}
Release:	%{rpmrelease}
Provides:	kernel-devel = %{kverrel}
Requires(pre):	%{kname}-%{kverrel}
Requires:	glibc-devel
Requires:  ncurses-devel
Requires:  make
Requires:  gcc >= 11.2.0
Requires:  perl
Summary:	The %{kname}-%{kverrel} kernel headers
Group:		System/Kernel and hardware
AutoReqProv:	no
%ifarch %{ix86}	
Conflicts:	arch(x86_64)
%endif
Provides:	devel(linux-vdso(64bit))

%description -n %{kname}-devel-%{kverrel}
This package contains the kernel header files for the linux kernel.
This is a stripped down version of the full kernel-source that only holds the
kernel headers, Makefiles, KConfig files and some needed binaries. This package
should be sufficient to build most external drivers against. If you want to build 
your own kernel from source, then you need to install the main kernel-source package.



###
# kernel-doc: documentation for the Linux kernel
###
%if %build_doc
%package -n	%{kname}-doc-%{kverrel}
Version:	%{rpmversionn}
Release:	%{rpmrelease}
Summary:	Various documentation found in the kernel source
Group:		System/Kernel and hardware
AutoReqProv: no

%description -n %{kname}-doc-%{kverrel}
This package contains only documentation files from the kernel source. Various
bits of information about the Linux kernel and the device drivers shipped
with it are documented in these files. Install this package if you need a reference to
the options that can be passed to kernel modules at load time. This package is NOT
required if you have the kernel-source rpm already installed.
%endif

#

###
# Apply our various patches
###
%prep
export USE_CCACHE=1

export CFLAGS="$CFLAGS -pipe -ffast-math"
export CXXFLAGS="$CXXFLAGS -pipe -ffast-math"


%setup -q -n %top_dir_name -c

cd %src_dir

%autopatch -p1

# get rid of unwanted files
find . -name '*~' -o -name '*.orig' -o -name '*.append' |xargs rm -f

# install the defconfig
install %SOURCE1 %{src_dir}/arch/x86/configs/x86_64_defconfig


###
# Kernel defconfig modifications for the various kernel builds
###

KERN_CONFIG=%{src_dir}/arch/x86/configs/%{target_arch}_defconfig

#enable kernel config function
enable_y()
{
perl -pi -e "s!# CONFIG_$1 is not set!CONFIG_$1=y!" $KERN_CONFIG
}

#disable kernel config function
disable()
{
perl -pi -e "s!CONFIG_$1=y!# CONFIG_$1 is not set!" $KERN_CONFIG
perl -pi -e "s!CONFIG_$1=m!# CONFIG_$1 is not set!" $KERN_CONFIG
}


# AMD64/Opteron x86 K8 kernel optimisation
%if %k8ops
    disable M686
    disable X86_GENERIC
    enable_y MK8
%endif

# enable voluntary preemption
%if %preempt_vol
    disable PREEMPT_NONE
    enable_y PREEMPT_VOLUNTARY
    enable_y PREEMPT_BKL
%endif

# enable full kernel preemption
%if %preempt_full
    disable PREEMPT_NONE
    enable_y PREEMPT
    enable_y PREEMPT_BKL
%endif

# enable PAE support
%if %pae_on
    disable HIGHMEM4G
    enable_y HIGHMEM64G
    enable_y X86_PAE
%endif

# enable HZ_100
%if %hz_100
    disable HZ_1000
    enable_y HZ_100
%endif

# enable HZ_250
%if %hz_250
    disable HZ_1000
    enable_y HZ_250
%endif

# enable xen guest support
%if %xen_guest
    enable_y XEN
%endif

###
# Prepare our kernel for building
###
%build

# common target directories
%define _kerneldir /usr/src/linux-%{kverrel}
%define _bootdir /boot
%define _modulesdir /lib/modules
%define _develdir /usr/src/%{kname}-devel-%{kverrel}

# directory definitions needed for building
%define temp_root %{build_dir}/temp-root
%define temp_source %{temp_root}%{_kerneldir}
%define temp_boot %{temp_root}%{_bootdir}
%define temp_modules %{temp_root}%{_modulesdir}
%define temp_devel %{temp_root}%{_develdir}

# set up our kernel name and release
cd %src_dir

%smake -s mrproper

# make sure kernel EXTRAVERSION and NAME say what we want it to say
%if ! %build_linus
#    %%if %%stabrel
#	LC_ALL=C perl -p -i -e "s/^EXTRAVERSION.*/EXTRAVERSION = .%%{stabrel}-%%{realrelease}/" Makefile
#    %%else
	LC_ALL=C perl -p -i -e "s/^EXTRAVERSION.*/EXTRAVERSION = -%{realrelease}/" Makefile
#    %%endif
    LC_ALL=C perl -p -i -e "s/^NAME.*/NAME =%{codename}/" Makefile
%endif

# fetch our kernel .config
cp arch/x86/configs/%{target_arch}_defconfig .config
%smake oldconfig



###
# Build our kernel
###

%kmake all
# start installing stuff
install -d %{temp_boot}
install -m 644 System.map %{temp_boot}/System.map-%{kverrel}
install -m 644 .config %{temp_boot}/config-%{kverrel}
cp -f arch/%{target_arch}/boot/bzImage %{temp_boot}/vmlinuz-%{kverrel}
# modules
install -d %{temp_modules}/%{kverrel}
%smake INSTALL_MOD_PATH=%{temp_root} KERNELRELEASE=%{kverrel} modules_install 

# remove firmware blobs as we provide an external kernel-firmware rpm
rm -rf %{temp_root}/lib/firmware

# create kernel rpm file list
output=../kernel_files.%{kverrel}
echo "%defattr(-,root,root)" > $output
echo "%{_bootdir}/config-%{kverrel}" >> $output
echo "%{_bootdir}/vmlinuz-%{kverrel}" >> $output
echo "%{_bootdir}/System.map-%{kverrel}" >> $output
echo "%dir %{_modulesdir}/%{kverrel}/" >> $output
echo "%{_modulesdir}/%{kverrel}/kernel" >> $output
echo "%{_modulesdir}/%{kverrel}/modules.*" >> $output



###
# Create kernel-devel rpm tree
###

mkdir -p %{temp_devel}
for i in $(find . -name 'Makefile*'); do cp -R --parents $i %{temp_devel};done
for i in $(find . -name 'Kconfig*' -o -name 'Kbuild*'); do cp -R --parents $i %{temp_devel};done
cp -fR include %{temp_devel}
cp -fR scripts %{temp_devel}
cp -fR arch/x86/kernel/asm-offsets.{c,s} %{temp_devel}/arch/x86/kernel/
cp -fR arch/x86/kernel/asm-offsets_{32,64}.c %{temp_devel}/arch/x86/kernel/
cp -fR arch/x86/include %{temp_devel}/arch/x86/

# needed for generation of kernel/bounds.s
cp -fR kernel/bounds.c %{temp_devel}/kernel/

# needed for lguest
#cp -fR drivers/lguest/lg.h %{temp_devel}/drivers/lguest/
cp -fR .config %{temp_devel}
cp -fR Module.symvers %{temp_devel}

# needed for truecrypt build
cp -fR drivers/md/dm.h %{temp_devel}/drivers/md/

# needed for external dvb tree
#cp -fR drivers/media/dvb-core/*.h %{temp_devel}/drivers/media/dvb-core/
cp -fR drivers/media/dvb-frontends/*.h %{temp_devel}/drivers/media/dvb-frontends/

# needed for fglrx build (2.6.29)
cp -fR drivers/acpi/acpica/*.h %{temp_devel}/drivers/acpi/acpica/

# aufs2 has a special file needed
# restore when aufs works again
cp -fR fs/aufs/magic.mk %{temp_devel}/fs/aufs

# needed for external mac80211 builds
cp -fR drivers/base/core.c %{temp_devel}/drivers/base/


cp -fR 3rdparty/rtl8821ce/halmac.mk %{temp_devel}/3rdparty/rtl8821ce/
cp -fR 3rdparty/rtl8821ce/rtl8821c.mk %{temp_devel}/3rdparty/rtl8821ce/
mkdir -p %{temp_devel}/3rdparty/rtl8821ce/hal/phydm/
cp -fR 3rdparty/rtl8821ce/hal/phydm/phydm.mk %{temp_devel}/3rdparty/rtl8821ce/hal/phydm/
mkdir -p %{temp_devel}/3rdparty/rtl8812au/hal/phydm/
cp -fR 3rdparty/rtl8812au/hal/phydm/phydm.mk %{temp_devel}/3rdparty/rtl8812au/hal/phydm/


# disable mrproper in -devel rpms
patch -p1 --fuzz=0 -d %{temp_devel} -i %{SOURCE2}

# Clean the scripts tree, and make sure everything is ok (sanity check)
# running prepare+scripts (tree was already "prepared" in build)
pushd %{temp_devel} >/dev/null
%smake -s prepare scripts
%smake -s clean
popd >/dev/null

rm -f %{temp_devel}/.config.old
rm -f %{temp_devel}/Makefile.*
rm -f %{temp_devel}/MAINTAINERS.*
rm -f %{temp_devel}/io_uring/Makefile

# clean out config split dirs
rm -f %{temp_devel}/include/config/*.{h,cmd}
find %{temp_devel}/include/config/* -type d -print | %kxargs /bin/rm -rf

# Fix permissions
chmod -R a+rX %{temp_devel}


# Prepare the kernel-sources for packaging
%if %build_source
    cd %src_dir
    %smake -s mrproper
%endif

###
# Install the beast
###
%install

cd %src_dir
# Directories definition needed for installing
%define target_source %{buildroot}/%{_kerneldir}
%define target_boot %{buildroot}%{_bootdir}
%define target_modules %{buildroot}%{_modulesdir}
%define target_devel %{buildroot}%{_develdir}

# We want to be able to test several times the install part
rm -rf %{buildroot}
cp -a %{temp_root} %{buildroot}

%if %build_source
    # create directories infastructure
    install -d %{target_source} 
    tar cf - . | tar xf - -C %{target_source}
    chmod -R a+rX %{target_source}
    # delete other misc files
    rm -f %{target_source}/{.clang-format,.gitattributes,.config.old,.config.cmd,.mailmap,.missing-syscalls.d,.gitignore,arch/.gitignore,firmware/.gitignore}
    cp -fR %{temp_devel}/.config %{target_source}
%endif

# we remove all the source files that we dont need
for i in alpha arm arm64 arc avr32 blackfin cris frv h8300 hexagon ia64 m32r m68k m68knommu metag microblaze \
mips mn10300 nds32 nios2 openrisc parisc powerpc riscv s390 score sh sparc tile unicore32 xtensa; do
    rm -rf %{target_source}/arch/$i
    rm -rf %{target_devel}/arch/$i
    rm -rf %{target_source}/include/asm-$i
    rm -rf %{target_devel}/include/asm-$i

# other misc files
rm -f %{target_source}/{.missing-syscalls.d,arch/.gitignore,firmware/.gitignore,.cocciconfig}

done

# compressing modules
find %{target_modules} -name "*.ko" | %kxargs xz -6e

for i in %{target_modules}/*; do rm -f $i/build $i/source;done

# we really need the depmod -ae here to make sure
pushd %{target_modules}
for i in *; do
    /sbin/depmod -u -ae -b %{buildroot} -r -F %{target_boot}/System.map-$i $i
    echo $?
done

#create our module descriptions
for i in *; do
    pushd $i
    echo "Creating module.description for $i"
    modules=`find . -name "*.ko.[g,x]z"`
    echo $modules | %kxargs /sbin/modinfo \
    | perl -lne 'print "$name\t$1" if $name && /^description:\s*(.*)/; $name = $1 if m!^filename:\s*(.*)\.k?o!; $name =~ s!.*/!!' > modules.description
    popd
done
popd

# clean patch letovers
rm -rf %buildroot/usr/src/linux-*/LICENSES
rm -rf %buildroot/usr/src/linux-*/MAINTAINERS.*
rm -rf %buildroot/usr/src/linux-*/Makefile.*

###
# Clean up
###
%if %dontclean
%clean
%else
%clean
rm -rf %{buildroot}
rm -rf %{build_dir}
%endif

%posttrans
# we don't do dkms build/install in init 1 because some other services can be affected in case dkms fails, so do related stuff NOW
SYSUSER=$(who |cut -d' ' -f1)
su $SYSUSER -c 'notify-send "Updating dkms kernel drivers if needed.
Please wait for the transaction to complete.
This can take a few minutes..." -t 30000'

DKMS_TOOL=/usr/sbin/dkms_autoinstaller
if [ -x $DKMS_TOOL ]; then
for i in $(/bin/ls -l /var/lib/dkms/ | awk '/^d/ {print $9}'); do
/usr/sbin/dkms --rpm_safe_upgrade build -m $i -v $(/usr/bin/rpm -q --qf "%%{VERSION}\n" dkms-$i)-$(/usr/bin/rpm -q --qf "%%{RELEASE}\n" dkms-$i) -k %{kverrel}
/usr/sbin/dkms --rpm_safe_upgrade install -m $i -v $(/usr/bin/rpm -q --qf "%%{VERSION}\n" dkms-$i)-$(/usr/bin/rpm -q --qf "%%{RELEASE}\n" dkms-$i) --force -k %{kverrel}
/sbin/depmod %{kverrel} &> /dev/null
done
fi
exit 0

###
# Pre and post install scripts
###
%preun
/sbin/installkernel -R %{kverrel}
exit 0

%post
#ALWAYS REQUIRED do depmod before installkernel to sync modules on target machine
/sbin/depmod %{kverrel} &> /dev/null
/sbin/installkernel %{kverrel}

%postun
/sbin/kernel_remove_initrd %{kverrel}

#clean up after oursleves
rm -rf /lib/modules/%{kverrel}
#dkms: if using dkms, get rid of old dkms kernel baggage
if [ -d /var/lib/dkms ]; then 
	%ifarch %{ix86}
	    find /var/lib/dkms -name kernel-%{kverrel}-i586 -type l -print | %kxargs /bin/rm -rf
	%else
	    find /var/lib/dkms -name kernel-%{kverrel}-x86_64 -type l -print | %kxargs /bin/rm -rf
	%endif
	find /var/lib/dkms -name %{kverrel} -type d -print | %kxargs /bin/rm -rf
fi



%if %build_source
%postun -n %{kname}-source-%{kverrel}
#clean up after oursleves
rm -rf %{_kerneldir}
exit 0
%endif


%post -n %{kname}-devel-%{kverrel}
rm -f /usr/src/linux
ln -snf %{_develdir} /usr/src/linux

# Please note: draktools is creating nasty soft links pointing to
# the incorrect kernel header dirs at kernel install time.
# We need to delete them here first, before re-creating the correct links.
for i in /lib/modules/%{kverrel}/{build,source}; do
    if [ -L $i ]; then
	rm -f $i
    fi
done

# Now re-create the correct links
ln -sf %{_develdir} /lib/modules/%{kverrel}/build
ln -sf %{_develdir} /lib/modules/%{kverrel}/source

%postun -n %{kname}-devel-%{kverrel}
if [ -L /usr/src/linux ]; then 
    if [ "$(readlink /usr/src/linux)" = "/usr/src/%{kname}-devel-%{kverrel}" ]; then
	rm -f /usr/src/linux
    fi
fi

# we need to delete <modules>/{build,source} at unsinstall
for i in /lib/modules/%{kverrel}/{build,source}; do
    if [ -L $i ]; then
	rm -f $i
    fi
done


#clean up after oursleves
rm -rf %{_develdir}
exit 0

###
# Package lists
###

# kernel
%files -f kernel_files.%{kverrel}

# full source
%if %build_source
%files -n %{kname}-source-%{kverrel}
%defattr(-,root,root)
%dir %{_kerneldir}
%dir %{_kerneldir}/arch
%dir %{_kerneldir}/include
%{_kerneldir}/.config
%{_kerneldir}/COPYING
%{_kerneldir}/CREDITS
%{_kerneldir}/Documentation
%{_kerneldir}/MAINTAINERS
%{_kerneldir}/Makefile
%{_kerneldir}/README
%{_kerneldir}/Kbuild
%{_kerneldir}/arch/x86
%{_kerneldir}/arch/um
%{_kerneldir}/block
%{_kerneldir}/crypto
%{_kerneldir}/drivers
%{_kerneldir}/fs
%{_kerneldir}/init
%{_kerneldir}/ipc
%{_kerneldir}/kernel
%{_kerneldir}/lib
%{_kerneldir}/mm
%{_kerneldir}/net
%{_kerneldir}/samples
%{_kerneldir}/security
%{_kerneldir}/scripts
%{_kerneldir}/sound
%{_kerneldir}/tools
%{_kerneldir}/usr
%{_kerneldir}/virt
%{_kerneldir}/Kconfig
%{_kerneldir}/arch/Kconfig
%{_kerneldir}/include/acpi
%{_kerneldir}/include/asm-generic
%{_kerneldir}/include/crypto
%{_kerneldir}/include/clocksource
%{_kerneldir}/include/drm
%{_kerneldir}/include/dt-bindings
%{_kerneldir}/include/keys
%{_kerneldir}/include/linux
%{_kerneldir}/include/math-emu
%{_kerneldir}/include/net
%{_kerneldir}/include/pcmcia
%{_kerneldir}/include/scsi
%{_kerneldir}/include/sound
%{_kerneldir}/include/video
%{_kerneldir}/include/media
%{_kerneldir}/include/rdma
%{_kerneldir}/include/trace
%{_kerneldir}/include/uapi
%{_kerneldir}/include/xen
%{_kerneldir}/include/misc
%{_kerneldir}/include/ras
%{_kerneldir}/arch/c6x/*
%{_kerneldir}/include/target
%{_kerneldir}/include/kvm
%{_kerneldir}/include/soc
%{_kerneldir}/.get_maintainer.ignore
%{_kerneldir}/certs
%{_kerneldir}/arch/csky
%{_kerneldir}/include/vdso
%{_kerneldir}/include/kunit
%{_kerneldir}/include/memory/
%{_kerneldir}/3rdparty
%endif


# kernel-devel
%files -n %{kname}-devel-%{kverrel}
%defattr(-,root,root)
%dir %{_develdir}
%dir %{_develdir}/arch
%dir %{_develdir}/include
%{_develdir}/.config
%{_develdir}/Documentation
%{_develdir}/Kbuild
%{_develdir}/Makefile
%{_develdir}/Module.symvers
%{_develdir}/arch/c6x
%{_develdir}/arch/Kconfig
%{_develdir}/arch/x86
%{_develdir}/arch/um
%{_develdir}/block
%{_develdir}/crypto
%{_develdir}/drivers
%{_develdir}/fs
%{_develdir}/include/acpi
%{_develdir}/include/asm-generic
%{_develdir}/include/clocksource
%{_develdir}/include/config
%{_develdir}/include/crypto
%{_develdir}/include/drm
%{_develdir}/include/dt-bindings
%{_develdir}/include/keys
%{_develdir}/include/linux
%{_develdir}/include/math-emu
%{_develdir}/include/net
%{_develdir}/include/pcmcia
%{_develdir}/include/ras
%{_develdir}/include/rdma
%{_develdir}/include/scsi
%{_develdir}/include/sound
%{_develdir}/include/trace
%{_develdir}/include/uapi
%{_develdir}/include/video
%{_develdir}/include/media
%{_develdir}/include/xen
%{_develdir}/include/misc
%{_develdir}/init
%{_develdir}/ipc
%{_develdir}/kernel
%{_develdir}/lib
%{_develdir}/mm
%{_develdir}/net
%{_develdir}/samples
%{_develdir}/scripts
%{_develdir}/security
%{_develdir}/sound
%{_develdir}/tools
%{_develdir}/usr
%{_develdir}/virt
%{_develdir}/include/generated
%{_develdir}/Kconfig
%{_develdir}/include/target
%{_develdir}/include/soc
%{_develdir}/include/kvm
%{_develdir}/certs
%{_develdir}/arch/csky
%{_develdir}/include/vdso
%{_develdir}/include/kunit
%{_develdir}/include/memory
%{_develdir}/3rdparty

%if %build_doc
#kernel documentation
%files -n %{kname}-doc-%{kverrel}
%defattr(-,root,root)
%doc linux-%{tar_vers}/Documentation/*
%endif


%changelog
